@@ -22,9 +22,6 @@
 #  Imports:
 #-------------------------------------------------------------------------------
 
-from enthought.traits.ui.extras.checkbox_column \
-    import CheckboxColumn
-
-from enthought.traits.ui.extras.edit_column \
-    import EditColumn
-
+from checkbox_column import CheckboxColumn
+from edit_column import EditColumn
+from saving import CanSaveMixin, SaveHandler
