@@ -23,10 +23,14 @@
 #-------------------------------------------------------------------------------
 
 import sys
+import glob
+from configobj import ConfigObj
+                
+from enthought.pyface.timer.api import do_later
 
 from enthought.traits.api \
     import HasTraits, HasPrivateTraits, Str, Instance, Property, Any, \
-           Code, HTML, true, false
+           Code, HTML, true, false, cached_property, Dict
            
 from enthought.traits.ui.api \
     import TreeEditor, ObjectTreeNode, TreeNodeObject, View, Item, VSplit, \
@@ -37,7 +41,7 @@ from os \
     import listdir
     
 from os.path \
-    import join, isdir, split, splitext, dirname, abspath
+    import join, isdir, split, splitext, dirname, basename, abspath, exists, isabs
     
 #-------------------------------------------------------------------------------
 #  Global data:  
@@ -68,13 +72,20 @@ def parse_source ( file_name ):
         fh.close()
     
         # Extract out the module comment as the description:
+        # FIXME: This isn't ideal: it will retrieve the first docstring found
+        # which might not be for the module as a whole. This needs to be
+        # improved later. For now, we want to make sure we catch docstrings
+        # even if there are comments etc. prior to them.
         comment = ''
-        quotes  = source[:3]
-        if (quotes == '"""') or (quotes == "'''"):
-            col = source.find( quotes, 3 )
-            if col >= 0:
-                comment = source[ 3: col ]
-                source  = source[ col + 3: ].strip()
+        quotes_styles = ["'''", '"""']
+        for quotes in quotes_styles:
+            start_index = source.find(quotes)
+            if start_index >= 0:
+                col = source.find( quotes, start_index + 3 )
+                if col >= 0:
+                    comment = source[ start_index + 3: col ]
+                    source  = source[: start_index].strip() + source[ col + 3: ].strip()
+                break 
                 
         return ( comment, source )
         
@@ -109,10 +120,15 @@ class DemoFileHandler ( Handler ):
         
         # Read in the demo source file:
         df.description, df.source = parse_source( df.path )
-        
         # Try to run the demo source file:
+
+        # Append the path for the demo source file to sys.path, so as to
+        # resolve any local (relative) imports in the demo source file.
+        sys.path.append(dirname(df.path))
+
         locals = df.parent.init_dic
         locals[ '__name__' ] = '___main___'
+        locals['__file__'] = df.path
         sys.modules[ '__main__' ].__file__ = df.path
         try:
             execfile( df.path, locals, locals )
@@ -125,11 +141,25 @@ class DemoFileHandler ( Handler ):
                     demo = DemoButton( demo = demo )
                 else:    
                     demo = self._get_object( 'demo', locals )
+            # FIXME: If a 'demo' object could not be found, then try to execute
+            # the file setting __name__ to __main__. A lot of test scripts have
+            # the actual test running when __name__==__main__ and so we can at
+            # least run all test examples this way. Use a do_later loop so as to
+            # finish building the current UI before running the test.
+            if demo is None:
+                locals['__name__'] = '__main__'
+                #do_later(self.execute_test, df, locals)
         except Exception, excp:
             demo = DemoError( msg = str( excp ) )
-            
+
+        # Clean up sys.path
+        sys.path.remove(dirname(df.path))
         df.demo = demo
-  
+    
+    def execute_test(self, df, locals):
+        """ Executes the file in df.path in the namespace of locals."""
+        execfile(df.path, locals, locals)
+        
     #---------------------------------------------------------------------------
     #  Closes the view:
     #---------------------------------------------------------------------------
@@ -230,8 +260,8 @@ class DemoButton ( HasPrivateTraits ):
             )
         ),
         resizable = True
-    ) 
-                        
+    )
+                   
 #-------------------------------------------------------------------------------
 #  'ModalDemoButton' class:  
 #-------------------------------------------------------------------------------
@@ -352,6 +382,7 @@ class DemoFile ( DemoTreeNodeObject ):
     # Log of all print messages displayed:
     log = Code
     
+    _nice_name = Str
     #---------------------------------------------------------------------------
     #  Implementation of the 'path' property:  
     #---------------------------------------------------------------------------
@@ -362,10 +393,17 @@ class DemoFile ( DemoTreeNodeObject ):
     #---------------------------------------------------------------------------
     #  Implementation of the 'nice_name' property:  
     #---------------------------------------------------------------------------
-                
+    
     def _get_nice_name ( self ):
-        return user_name_for( self.name )
-        
+        if not self._nice_name:
+            self._nice_name = user_name_for( self.name )
+        return self._nice_name
+    
+    def _set_nice_name(self, value):
+        old = self.nice_name
+        self._nice_name = value
+        self.trait_property_changed('nice_name', old, value)
+    
     #---------------------------------------------------------------------------
     #  Returns whether or not the object has children:  
     #---------------------------------------------------------------------------
@@ -412,6 +450,16 @@ class DemoPath ( DemoTreeNodeObject ):
     # Paths do allow children:
     allows_children = true
     
+    # Configuration dictionary for this node
+    # This trait is set when a config file exists for the parent of this path.
+    config_dict = Dict
+    
+    # Configuration file for this node.
+    config_filename = Str
+    
+    # Cached value of the nice_name property.
+    _nice_name = Str
+    
     #---------------------------------------------------------------------------
     #  Implementation of the 'path' property:  
     #---------------------------------------------------------------------------
@@ -422,9 +470,20 @@ class DemoPath ( DemoTreeNodeObject ):
     #---------------------------------------------------------------------------
     #  Implementation of the 'nice_name' property:  
     #---------------------------------------------------------------------------
-                
+    
     def _get_nice_name ( self ):
-        return user_name_for( self.name )
+        if not self._nice_name:
+            self._nice_name = user_name_for( self.name )
+        return self._nice_name
+        
+    #---------------------------------------------------------------------------
+    #  Setter for the 'nice_name' property:  
+    #---------------------------------------------------------------------------
+    
+    def _set_nice_name(self, value):
+        old = self.nice_name
+        self._nice_name = value
+        self.trait_property_changed('nice_name', old, value)
         
     #---------------------------------------------------------------------------
     #  Implementation of the 'description' property:  
@@ -454,7 +513,6 @@ class DemoPath ( DemoTreeNodeObject ):
         init_dic = {}
         description, source = parse_source( join( self.path, '__init__.py' ) )
         exec (exec_str + source) in init_dic
-        
         return init_dic
         
         # fixme: The following code should work, but doesn't, so we use the
@@ -510,6 +568,19 @@ class DemoPath ( DemoTreeNodeObject ):
     def get_children ( self ):
         """ Gets the object's children.
         """
+        if self.config_dict or self.config_filename:
+            children = self.get_children_from_config()
+        else:
+            children = self.get_children_from_datastructure()
+        return children
+    
+    #---------------------------------------------------------------------------
+    #  Gets the object's children based on the filesystem structure.
+    #---------------------------------------------------------------------------
+    def get_children_from_datastructure( self ):
+        """ Gets the object's children based on the filesystem structure.
+        """    
+        
         dirs  = []
         files = []
         path  = self.path
@@ -529,6 +600,68 @@ class DemoPath ( DemoTreeNodeObject ):
         
         return (dirs + files)
         
+    #---------------------------------------------------------------------------
+    # Gets the object's children as specified in its configuration file or 
+    # dictionary.  
+    #---------------------------------------------------------------------------
+
+    def get_children_from_config( self ):
+        """
+        Gets the object's children as specified in its configuration file or 
+        dictionary.
+        """
+        
+        if not self.config_dict:
+            if exists(self.config_filename):
+                try:
+                    self.config_dict = ConfigObj(self.config_filename)
+                except:
+                    pass
+        if not self.config_dict:
+            return self.get_children_from_datastructure()
+        
+        dirs = []
+        files = []
+        for keyword, value in self.config_dict.items():
+            if not value.get('no_demo'):
+                sourcedir = value.pop('sourcedir', None)
+                if sourcedir is not None:
+                    # This is a demo directory.
+                    demoobj = DemoPath( parent = self, name = sourcedir )
+                    demoobj.nice_name = keyword
+                    demoobj.config_dict = value
+                    dirs.append(demoobj)
+                else:
+                    names = []
+                    filenames = value.pop('files', [])
+                    if not isinstance(filenames, list):
+                        filenames = [filenames]
+                    for filename in filenames:
+                        filename = join(self.path, filename)
+                        for name in glob.iglob(filename):
+                            pathname, ext = splitext(name)
+                            if (ext == '.py') and \
+                                (basename(pathname) != '__init__'):
+                                names.append(pathname)
+                    if len(names) > 1:
+                        config_dict = {}
+                        for name in names:
+                            config_dict[basename(name)] = {'files': 
+                                                           name + '.py'}
+                        demoobj = DemoPath( parent = self, name = '')
+                        demoobj.nice_name = keyword
+                        demoobj.config_dict = config_dict
+                        dirs.append(demoobj)
+                    elif len(names) == 1:
+                        file = DemoFile(parent=self, name=names[0])
+                        file.nice_name = keyword
+                        files.append(file)
+                    
+        dirs.sort( lambda l, r: cmp( l.nice_name, r.nice_name ) )
+        files.sort( lambda l, r: cmp( l.nice_name, r.nice_name ) )
+        
+        return (dirs + files)
+        
     #---------------------------------------------------------------------------
     #  Returns whether the specified path contains any .py files:  
     #---------------------------------------------------------------------------
@@ -587,7 +720,11 @@ demo_view = View(
                    label      = 'Demo',
                    show_label = False,
                    style      = 'custom',
-                   resizable  = True
+                   resizable  = True,
+                   # FIXME:
+                   # visible_when doesn't work correctly yet (for wx atleast) 
+                   # for tabbed items. Needs more investigation.
+                   visible_when = 'demo',
              ),
              export = 'DockWindowShell'
          ),
@@ -633,24 +770,32 @@ class Demo ( HasPrivateTraits ):
     # Root path object for locating demo files:
     root = Instance( DemoPath )
         
+    # Title for the demo
+    title = Str
+
     #---------------------------------------------------------------------------
     #  Traits view definitions:  
     #---------------------------------------------------------------------------
         
-    traits_view = View( 
-        Item( name       = 'root',
-              id         = 'root',
-              show_label = False,
-              editor     = demo_tree_editor
-        ),
-        title     = 'Traits UI Demo',
-        id        = 'enthought.traits.ui.demos.demo.Demo',
-        dock      = 'horizontal',
-        resizable = True,
-        width     = 950,
-        height    = 600
-    )
                         
+    def default_traits_view(self):
+        """ Constructs the default traits view."""
+        
+        traits_view = View( 
+            Item( name       = 'root',
+                  id         = 'root',
+                  show_label = False,
+                  editor     = demo_tree_editor
+            ),
+            title     = self.title,
+            id        = 'enthought.traits.ui.demos.demo.Demo',
+            dock      = 'horizontal',
+            resizable = True,
+            width     = 950,
+            height    = 600
+        )
+        return traits_view
+    
     #---------------------------------------------------------------------------
     #  Handles the 'root' trait being changed:  
     #---------------------------------------------------------------------------
@@ -664,9 +809,17 @@ class Demo ( HasPrivateTraits ):
 #  Function to run the demo:
 #-------------------------------------------------------------------------------
         
-def demo ( ):
-    path, name = split( dirname( abspath( sys.argv[0] ) ) )
+def demo ( use_files=False, dir_name = None, config_filename = '', 
+           title = 'Traits UI Demos' ):
+    if dir_name is None:
+        dir_name = dirname(abspath( sys.argv[0] ))
+    path, name = split( dir_name )
+    if len(config_filename) > 0 and not isabs(config_filename):
+        config_filename = join(path, name, config_filename)
     Demo( path = path, 
-          root = DemoPath( name = name, use_files = False )
+          title = title,
+          root = DemoPath( name = name, 
+                           use_files = use_files,
+                           config_filename = config_filename )
     ).configure_traits() 
     
