@@ -204,7 +204,7 @@ class TabularAdapter ( HasPrivateTraits ):
     # *columns* trait):
     column_map = Property( depends_on = 'columns' )
     
-    # The mapping from column indices to column labels (defined by the *columns* 
+    # The mapping from column indices to column labels (defined by the *columns*
     # trait):
     label_map = Property( depends_on = 'columns' )
     
@@ -303,11 +303,11 @@ class TabularAdapter ( HasPrivateTraits ):
         """
         return self._result_for( 'get_content', object, trait, row, column )
      
-    def set_text ( self, object, trait, row, text ):
+    def set_text ( self, object, trait, row, column, text ):
         """ Sets the text for a specified *object.trait[row].column* item to
             *text*.
         """
-        self._result_for( 'set_text', object, trait, row, 0, text )
+        self._result_for( 'set_text', object, trait, row, column, text )
         
     def get_tooltip ( self, object, trait, row, column ):
         """ Returns the tooltip for a specified row.
@@ -372,11 +372,17 @@ class TabularAdapter ( HasPrivateTraits ):
             self.object, self.name, self.row, self.column ) % self.get_content( 
             self.object, self.name, self.row, self.column )
      
-    def _set_text ( self ):
+    def _set_text ( self, value ):
         if isinstance( self.column_id, int ):
             self.item[ self.column_id ] = self.value
-        else:    
-            setattr( self.item, self.column_id, self.value )
+        else:
+            # Convert value to the correct trait type.
+            try:
+                trait_handler = self.item.trait(self.column_id).handler
+                setattr( self.item, self.column_id, 
+                                    trait_handler.evaluate(self.value))
+            except:
+                setattr( self.item, self.column_id, value )
         
     def _get_content ( self ):
         if isinstance( self.column_id, int ):
@@ -419,7 +425,7 @@ class TabularAdapter ( HasPrivateTraits ):
                     label = label[0]
                     
                 indices.append( labels.index( label ) )
-                
+            map.append(indices)    
         return map
                 
     @cached_property
@@ -491,7 +497,7 @@ class TabularAdapter ( HasPrivateTraits ):
             if item is not None:
                 for klass in item_class.__mro__:
                     handler = (self._get_handler_for( '%s_%s_%s' % 
-                          ( klass.__name__, column_id, trait_name ), prefix ) or 
+                          ( klass.__name__, column_id, trait_name ), prefix ) or
                         self._get_handler_for( '%s_%s' % 
                           ( klass.__name__, trait_name ), prefix ))
                     if handler is not None:
@@ -500,7 +506,7 @@ class TabularAdapter ( HasPrivateTraits ):
             if handler is None: 
                 handler = (self._get_handler_for( '%s_%s' % ( column_id, 
                               trait_name ), prefix ) or 
-                           self._get_handler_for( trait_name, 'get_' ))
+                           self._get_handler_for( trait_name, prefix ))
                            
         self.cache[ key ] = handler
         return handler()
