@@ -323,6 +323,16 @@ class Toolkit ( HasPrivateTraits ):
         """
         raise NotImplementedError
 
+    #---------------------------------------------------------------------------
+    #  Indicates that an event should continue to be processed by the toolkit
+    #---------------------------------------------------------------------------
+
+    def skip_event ( self, event ):
+        """ Indicates that an event should continue to be processed by the 
+            toolkit.
+        """
+        raise NotImplementedError
+
     #---------------------------------------------------------------------------
     #  Destroys a specified GUI toolkit control:  
     #---------------------------------------------------------------------------
@@ -361,6 +371,7 @@ class Toolkit ( HasPrivateTraits ):
         """ Returns a dictionary of useful constants.
         
             Currently, the dictionary should have the following key/value pairs:
+                
             - WindowColor': the standard window background color in the toolkit
               specific color format.
         """
